# copyright 2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-narval postcreate script, executed at instance creation time or when
the cube is added to an existing instance.

You could setup site properties or a workflow here for example.
"""
_ = unicode

# create the WF of Plan
wf = add_workflow(u'Plan execution status workflow', 'Plan')

ready = wf.add_state(_('ready'), initial=True)
running = wf.add_state(_('running'))
done = wf.add_state(_('done'))
error = wf.add_state(_('error'), description=_('The execution of the Plan has been terminated (due to unexpected error)'))
killed = wf.add_state(_('killed'), description=_('The execution of the Plan has been terminated (due to resource exhaustion)'))
wf.add_transition(_('start'), ready, running, requiredgroups=('narval','managers'))
wf.add_transition(_('end'), running, done, requiredgroups=('narval','managers'))
wf.add_transition(_('fail'), running, error, requiredgroups=('narval','managers'))
wf.add_transition(_('kill'), running, killed, requiredgroups=('narval','managers'))

# create a dumb test recipe
recipe = create_entity('Recipe', name=u'test.noop', script=u'pass')

